# Generated by roxygen2: do not edit by hand

export(identifyLCAMixture)
export(identifyMixture)
export(plotBubble)
export(plotScatter)
export(priorOnAlpha_spec)
export(priorOnE0_spec)
export(priorOnK_spec)
export(sampleLCA)
export(sampleLCAMixture)
export(sampleMultNormMixture)
export(samplePoisMixture)
export(sampleUniNormMixture)
importFrom(extraDistr,dbnbinom)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,axis)
importFrom(graphics,par)
importFrom(graphics,symbols)
importFrom(methods,is)
importFrom(mvtnorm,dmvnorm)
importFrom(stats,df)
importFrom(stats,dgamma)
importFrom(stats,dgeom)
importFrom(stats,dnbinom)
importFrom(stats,dpois)
importFrom(stats,kmeans)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
