% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimData.R
\docType{data}
\name{SimData}
\alias{SimData}
\title{Simulated Multivariate Binary Data}
\format{
A data frame with 500 observations and 31 variables:
\describe{
\item{y1}{binary variable coded 1 and 2}
\item{y2}{binary variable coded 1 and 2}
\item{y3}{binary variable coded 1 and 2}
\item{y4}{binary variable coded 1 and 2}
\item{y5}{binary variable coded 1 and 2}
\item{y6}{binary variable coded 1 and 2}
\item{y7}{binary variable coded 1 and 2}
\item{y8}{binary variable coded 1 and 2}
\item{y9}{binary variable coded 1 and 2}
\item{y10}{binary variable coded 1 and 2}
\item{y11}{binary variable coded 1 and 2}
\item{y12}{binary variable coded 1 and 2}
\item{y13}{binary variable coded 1 and 2}
\item{y14}{binary variable coded 1 and 2}
\item{y15}{binary variable coded 1 and 2}
\item{y16}{binary variable coded 1 and 2}
\item{y17}{binary variable coded 1 and 2}
\item{y18}{binary variable coded 1 and 2}
\item{y19}{binary variable coded 1 and 2}
\item{y20}{binary variable coded 1 and 2}
\item{y21}{binary variable coded 1 and 2}
\item{y22}{binary variable coded 1 and 2}
\item{y23}{binary variable coded 1 and 2}
\item{y24}{binary variable coded 1 and 2}
\item{y25}{binary variable coded 1 and 2}
\item{y26}{binary variable coded 1 and 2}
\item{y27}{binary variable coded 1 and 2}
\item{y28}{binary variable coded 1 and 2}
\item{y29}{binary variable coded 1 and 2}
\item{y30}{binary variable coded 1 and 2}
\item{z}{integer variable with values 1, 2, and 3 indicating group membership}
}
}
\description{
Simulated multivariate binary data with a 3-group
structure where the variables are correlated within the groups.
}
\keyword{data}
